package sun.security.rsa;

import java.io.ObjectStreamException;
import java.security.KeyRep;
import sun.security.util.DerInputStream;
import java.security.ProviderException;
import java.io.IOException;
import sun.security.util.BitArray;
import sun.security.util.DerValue;
import sun.security.util.DerOutputStream;
import sun.security.x509.AlgorithmId;
import java.security.InvalidKeyException;
import java.security.spec.AlgorithmParameterSpec;
import java.math.BigInteger;
import java.security.interfaces.RSAPublicKey;
import sun.security.x509.X509Key;

public final class RSAPublicKeyImpl extends X509Key implements RSAPublicKey
{
    private static final long serialVersionUID = 2644735423591199609L;
    private static final BigInteger THREE;
    private BigInteger n;
    private BigInteger e;
    private AlgorithmParameterSpec keyParams;
    
    public static RSAPublicKey newKey(byte[] array) throws InvalidKeyException {
        return new RSAPublicKeyImpl(array);
    }
    
    public static RSAPublicKey newKey(RSAUtil.KeyType keyType, AlgorithmParameterSpec algorithmParameterSpec, BigInteger bigInteger, BigInteger bigInteger2) throws InvalidKeyException {
        return new RSAPublicKeyImpl(RSAUtil.createAlgorithmId(keyType, algorithmParameterSpec), bigInteger, bigInteger2);
    }
    
    RSAPublicKeyImpl(AlgorithmId algid, BigInteger n, BigInteger e) throws InvalidKeyException {
        RSAKeyFactory.checkRSAProviderKeyLengths(n.bitLength(), e);
        checkExponentRange(n, e);
        this.n = n;
        this.e = e;
        this.keyParams = RSAUtil.getParamSpec(algid);
        this.algid = algid;
        try {
            DerOutputStream derOutputStream = new DerOutputStream();
            derOutputStream.putInteger(n);
            derOutputStream.putInteger(e);
            byte[] byteArray = new DerValue((byte)48, derOutputStream.toByteArray()).toByteArray();
            this.setKey(new BitArray(byteArray.length * 8, byteArray));
        }
        catch (IOException cause) {
            throw new InvalidKeyException(cause);
        }
    }
    
    RSAPublicKeyImpl(byte[] encodedKey) throws InvalidKeyException {
        if (encodedKey == null || encodedKey.length == 0) {
            throw new InvalidKeyException("Missing key encoding");
        }
        this.decode(encodedKey);
        RSAKeyFactory.checkRSAProviderKeyLengths(this.n.bitLength(), this.e);
        checkExponentRange(this.n, this.e);
        try {
            this.keyParams = RSAUtil.getParamSpec(this.algid);
        }
        catch (ProviderException cause) {
            throw new InvalidKeyException(cause);
        }
        return;
    }
    
    static void checkExponentRange(BigInteger val, BigInteger bigInteger) throws InvalidKeyException {
        if (bigInteger.compareTo(val) >= 0) {
            throw new InvalidKeyException("exponent is larger than modulus");
        }
        if (bigInteger.compareTo(RSAPublicKeyImpl.THREE) < 0) {
            throw new InvalidKeyException("exponent is smaller than 3");
        }
        return;
    }
    
    @Override
    public String getAlgorithm() {
        return this.algid.getName();
    }
    
    @Override
    public BigInteger getModulus() {
        return this.n;
    }
    
    @Override
    public BigInteger getPublicExponent() {
        return this.e;
    }
    
    @Override
    public AlgorithmParameterSpec getParams() {
        return this.keyParams;
    }
    
    @Override
    protected void parseKeyBits() throws InvalidKeyException {
        try {
            DerValue derValue = new DerInputStream(this.getKey().toByteArray()).getDerValue();
            if (derValue.tag != 48) {
                throw new IOException("Not a SEQUENCE");
            }
            DerInputStream data = derValue.data;
            this.n = data.getPositiveBigInteger();
            this.e = data.getPositiveBigInteger();
            if (derValue.data.available() != 0) {
                throw new IOException("Extra data available");
            }
        }
        catch (IOException cause) {
            throw new InvalidKeyException("Invalid RSA public key", cause);
        }
    }
    
    @Override
    public String toString() {
        return "Sun " + this.getAlgorithm() + " public key, " + this.n.bitLength() + " bits\n  params: " + this.keyParams + "\n  modulus: " + this.n + "\n  public exponent: " + this.e;
    }
    
    protected Object writeReplace() throws ObjectStreamException {
        return new KeyRep(KeyRep.Type.PUBLIC, this.getAlgorithm(), this.getFormat(), this.getEncoded());
    }
    
    static {
        THREE = BigInteger.valueOf(3L);
    }
}
